using System;

using System.Runtime.InteropServices;

using PVOID = System.IntPtr;
using DWORD = System.UInt32;


namespace usb_api
{
	unsafe public class usb_interface
	{
		#region  String Definitions of Pipes and VID_PID
		//string vid_pid_boot= "vid_04d8&pid_000b";    // Bootloader vid_pid ID
		string vid_pid_norm= "vid_04d8&pid_000c";

		string out_pipe= "\\MCHP_EP1"; // Define End Points
		string in_pipe= "\\MCHP_EP1";
		#endregion

		#region Imported DLL functions from mpusbapi.dll
		[DllImport("mpusbapi.dll")]
		private static extern DWORD _MPUSBGetDLLVersion();
		[DllImport("mpusbapi.dll")]
		private static extern DWORD _MPUSBGetDeviceCount(string pVID_PID);
		[DllImport("mpusbapi.dll")]
		private static extern IntPtr _MPUSBOpen(DWORD instance,string pVID_PID,string pEP,DWORD dwDir,DWORD dwReserved);
		[DllImport("mpusbapi.dll")]
        private static extern DWORD _MPUSBRead(IntPtr handle, IntPtr pData, DWORD dwLen, DWORD* pLength, DWORD dwMilliseconds);
		[DllImport("mpusbapi.dll")]
		private static extern DWORD _MPUSBWrite(IntPtr handle,IntPtr pData,DWORD dwLen,DWORD* pLength,DWORD dwMilliseconds);
		[DllImport("mpusbapi.dll")]
		private static extern DWORD _MPUSBReadInt(IntPtr handle,DWORD dwLen,DWORD* pLength,DWORD dwMilliseconds);
		[DllImport("mpusbapi.dll")]
		private static extern bool _MPUSBClose(IntPtr handle);
		#endregion

        #region Imported from kernel32
        [DllImport("kernel32.dll")]
        extern static IntPtr GlobalAlloc(uint uFlags, uint dwBytes);
        // HGLOBAL GlobalAlloc(UINT uFlags, SIZE_T dwBytes);

        [DllImport("kernel32.dll")]
        extern static IntPtr GlobalFree(IntPtr hMem);
        
        private enum   GlobalAllocFlags
            {
            GMEM_FIXED    = 0x0000,//Allocates fixed memory.
            GMEM_MOVEABLE = 0x0002,// Allocates movable memory.
            GMEM_ZEROINIT = 0x0040,// Initializes memory contents to 0.

            GPTR = 0x0040 ,//Combines GMEM_FIXED and GMEM_ZEROINIT.
            GHND = 0x0042 //Combines GMEM_MOVEABLE and GMEM_ZEROINIT.
        }
        #endregion

        //void* myOutPipe;
		//void* myInPipe;

        IntPtr myOutPipe;
        IntPtr myInPipe;

		private void OpenPipes()
		{
			DWORD selection=0; // Selects the device to connect to, in this example it is assumed you will only have one device per vid_pid connected.

			myOutPipe = _MPUSBOpen(selection,vid_pid_norm,out_pipe,0,0);
			myInPipe = _MPUSBOpen(selection,vid_pid_norm,in_pipe,1,0);
		}
		private void ClosePipes()
		{
			_MPUSBClose(myOutPipe);
			_MPUSBClose(myInPipe);
		}

		private DWORD SendReceivePacket(IntPtr SendData, DWORD SendLength, IntPtr ReceiveData, DWORD *ReceiveLength)
		{
            byte sendData = (byte)Marshal.PtrToStructure(SendData, typeof(byte));

			uint SendDelay=1000;
			uint ReceiveDelay=1000;

			DWORD SentDataLength;
			DWORD ExpectedReceiveLength = *ReceiveLength;

			OpenPipes();

				if(_MPUSBWrite(myOutPipe,SendData,SendLength,&SentDataLength,SendDelay)==1)
				{

					if(_MPUSBRead(myInPipe,ReceiveData, ExpectedReceiveLength,ReceiveLength,ReceiveDelay)==1)
					{
						if(*ReceiveLength == ExpectedReceiveLength)
						{
							ClosePipes();
							return 1;   // Success!
						}
						else if(*ReceiveLength < ExpectedReceiveLength)
						{
							ClosePipes();
							return 2;   // Partially failed, incorrect receive length
						}
					}
				}
			ClosePipes();
			return 0;  // Operation Failed
		}

		public DWORD GetDLLVersion()
		{
			return _MPUSBGetDLLVersion();
		}
		public DWORD GetDeviceCount(string Vid_Pid)
		{
			return _MPUSBGetDeviceCount(Vid_Pid);
		}

        private byte[] PtrToByteArray(IntPtr pointer) 
        {
            byte[] rude = new byte[64];
            Marshal.PtrToStructure(pointer, rude);

            return rude;
        }

        private void ByteArrayToPtr(Byte[] array, IntPtr pointer)
        {
            Marshal.StructureToPtr(array, pointer, false);
        }


        

		public int UpdateLED(uint led, bool State)
		{
			// The default demo firmware application has a defined application
			// level protocol.
			// To set the LED's, the host must send the UPDATE_LED
			// command which is defined as 0x32, followed by the LED to update,
			// then the state to chance the LED to.
			//
			// i.e. <UPDATE_LED><0x01><0x01>
			//
			// Would activate LED 1
			//
			// The receive buffer size must be equal to or larger than the maximum
			// endpoint size it is communicating with. In this case, it is set to 64 bytes.

            // byte* send_buf = stackalloc byte[64];
			byte[] receive_buf=null;
            byte[] send_buf = null;

            int result = 0;
            //byte[] send_buf = new byte[64];

            //Allocate memory
            IntPtr send_buf_ptr = GlobalAlloc((uint)GlobalAllocFlags.GMEM_ZEROINIT, 64);
            IntPtr rec_buf_ptr = GlobalAlloc((uint)GlobalAllocFlags.GMEM_ZEROINIT, 64);

            //Convert newly allocated memory into managed type
            send_buf = PtrToByteArray(send_buf_ptr);
           
                //Marshal.StructureToPtr(send_buf,
           

			DWORD RecvLength=3;
			send_buf[0] = 0x32;			//Command for LED Status  
			send_buf[1] = (byte)led;
			send_buf[2] = (byte)(State?1:0);

            //Convert back to ptr
            ByteArrayToPtr(send_buf, send_buf_ptr);
	


			if (SendReceivePacket(send_buf_ptr,3,rec_buf_ptr,&RecvLength) == 1)
			{
				if (RecvLength == 1 && receive_buf[0] == 0x32)
				{
                    result = 0;
				}
				else
				{
                    result = 2;
				}
			}
			else
			{
                result = 1;
			}
            //Clean up
            GlobalFree(send_buf_ptr);
            GlobalFree(rec_buf_ptr);

            return result;
		}

		public uint Add(uint num1, uint num2)
		{
			// The default demo firmware application has a defined application
			// level protocol.
			// To calculate the sum the host must send the DO_SUM
			// command which is defined as 0xEE, followed by the LED to update,
			// then the state to chance the LED to.
			//
			// i.e. <DO_SUM><0x01><0x02>
			//
			// Would return a command
			//
			// <DO_SUM><0x03>
			//
			// Bear in mind no error correction to prevent roll over in the PIC has 
			// been implemented, to keep this example as simple as possible.
			//
			// The receive buffer size must be equal to or larger than the maximum
			// endpoint size it is communicating with. In this case, it is set to 64 bytes.

            // byte* send_buf = stackalloc byte[64];
            byte[] receive_buf = null;
            byte[] send_buf = null;

            //Allocate memory
            IntPtr send_buf_ptr = GlobalAlloc((uint)GlobalAllocFlags.GMEM_ZEROINIT, 64);
            IntPtr rec_buf_ptr = GlobalAlloc((uint)GlobalAllocFlags.GMEM_ZEROINIT, 64);

            //Convert newly allocated memory into managed type
            send_buf = PtrToByteArray(send_buf_ptr);

			uint Temp=0;

			DWORD RecvLength=(byte)2;
			send_buf[0] = 0xEE;			//Command for DO_SUM  
			send_buf[1] = (byte)num1;
			send_buf[2] = (byte)num2;

            //Convert back to ptr
            ByteArrayToPtr(send_buf, send_buf_ptr);
            
            if (SendReceivePacket(send_buf_ptr, 4, rec_buf_ptr, &RecvLength) == 1)
            {

                //Convert  allocated memory into managed type
                receive_buf = PtrToByteArray(send_buf_ptr);

                Temp = receive_buf[1];

            
            }

            //Clean up
            GlobalFree(send_buf_ptr);
            GlobalFree(rec_buf_ptr);
		
			return Temp;
		}
	}

}
