/*********************************************************************
 *
 *                Microchip USB C18 Firmware Version 1.0
 *
 *********************************************************************
 * FileName:        main.c
 * Dependencies:    See INCLUDES section below
 * Processor:       PIC18
 * Compiler:        C18 2.30.01+
 * Company:         Microchip Technology, Inc.
 *
 * Software License Agreement
 *
 * The software supplied herewith by Microchip Technology Incorporated
 * (the Company) for its PICmicro Microcontroller is intended and
 * supplied to you, the Companys customer, for use solely and
 * exclusively on Microchip PICmicro Microcontroller products. The
 * software is owned by the Company and/or its supplier, and is
 * protected under applicable copyright laws. All rights are reserved.
 * Any use in violation of the foregoing restrictions may subject the
 * user to criminal sanctions under applicable laws, as well as to
 * civil liability for the breach of the terms and conditions of this
 * license.
 *
 * THIS SOFTWARE IS PROVIDED IN AN AS IS CONDITION. NO WARRANTIES,
 * WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
 * TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE COMPANY SHALL NOT,
 * IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
 * CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 * Author               Date        Comment
 *~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * Rawin Rojvanit       11/19/04    Original.
 ********************************************************************/

/** I N C L U D E S **********************************************************/
#include <p18cxxx.h>
#include "system\typedefs.h"                        // Required
#include "system\usb\usb.h"                         // Required
#include "io_cfg.h"                                 // Required
#include "pindefs.h"
#include "system\usb\usb_compile_time_validation.h" // Optional
#include "user\user.h"    
#include "lcd.h"   

                 // Modifiable

/** V A R I A B L E S ********************************************************/
#pragma udata

/** P R I V A T E  P R O T O T Y P E S ***************************************/
static void InitializeSystem(void);
void USBTasks(void);
void SetupPorts(void);
void CheckBus(void);
void CheckBattery(void);
void Delay(int);
void Beep(int);
/** V E C T O R  R E M A P P I N G *******************************************/

extern void _startup (void);        // See c018i.c in your C18 compiler dir
#pragma code _RESET_INTERRUPT_VECTOR = 0x000800
void _reset (void)
{
    _asm goto _startup _endasm
}
#pragma code

#pragma code _HIGH_INTERRUPT_VECTOR = 0x000808
void _high_ISR (void)
{
    ;
}

#pragma code _LOW_INTERRUPT_VECTOR = 0x000818
void _low_ISR (void)
{
    ;
}
#pragma code





/** D E C L A R A T I O N S **************************************************/
#pragma code
//Is usb connected
BOOL usb;
BOOL battery;



/******************************************************************************
 * Function:        void main(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        Main program entry point.
 *
 * Note:            None
 *****************************************************************************/
void main(void)
{

	
	int delay=0;

char str = 'f';
	

SetupPorts();

usb = PIN_USBPOWER;

ScreenOff();

Beep(5);
Delay(delay);
Beep(4);
Delay(delay);
ScreenOn();
Beep(3);
Delay(delay);
Beep(2);
Delay(delay);
Beep(1);



OpenLCD();
	
while(1)
{
	CheckBus();
	CheckBattery();
}


//do nothing




}//end main


void CheckBus()
{
	//Check if usb on var set
	if (usb)
	{
		//Check if usb is still connected
		if (!PIN_USBPOWER)
		{
			//Not connected anymore
			Beep(1);
			usb = 0;
		}
	}
	else
	{
		//Check if usb is connected now
		if (PIN_USBPOWER)
		{
			usb = 1;
			Beep (2);
		}	
	}
	
}


void CheckBattery()
{
	//Check if usb on var set
	if (battery)
	{
		//Check if usb is still connected
		if (!PIN_SELFPOWER)
		{
			//Not connected anymore
			Beep(1);
			battery = 0;
		}
	}
	else
	{
		//Check if usb is connected now
		if (PIN_SELFPOWER)
		{
			battery = 1;
			Beep (2);
		}	
	}
	
}





void SetupPorts(void)
{

	LATD =0;
	TRISB = 0x38;
	TRISC =0;
	TRISD=0xFF;
	TRISA = 0;
	TRISE =0;

	LATA =0;
	//Turn off comparators
//	ADCON0 = 0;
	ADCON1 = 0x0F;
	CMCON	= 0x07;

	LATB = 0xff;
	LATC =0xff;

	LATA = 0xff;
	LATE =0xff;
}

void Delay(int loops)
{
int i;
	
for (i=0;i<loops;i++)
{
	LATCbits.LATC0 = LATCbits.LATC0;

}
}


void Beep(int delay)
{
int A=75;
	LATCbits.LATC0 = 1;
	LATCbits.LATC1 = 0;

while (A >= 0)
{
	LATCbits.LATC0 = !LATCbits.LATC0;
	Delay(delay);
	
	LATCbits.LATC1 = !LATCbits.LATC1;
//	Delay(0);
A--;
}	
	
}

/******************************************************************************
 * Function:        static void InitializeSystem(void)
 *
 * PreCondition:    None
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        InitializeSystem is a centralize initialization routine.
 *                  All required USB initialization routines are called from
 *                  here.
 *
 *                  User application initialization routine should also be
 *                  called from here.                  
 *
 * Note:            None
 *****************************************************************************/
static void InitializeSystem(void)
{
    ADCON1 |= 0x0F;                 // Default all pins to digital
    
    #if defined(USE_USB_BUS_SENSE_IO)
    tris_usb_bus_sense = INPUT_PIN; // See io_cfg.h
    #endif
    
    #if defined(USE_SELF_POWER_SENSE_IO)
    tris_self_power = INPUT_PIN;
    #endif
    
    mInitializeUSBDriver();         // See usbdrv.h
    
    UserInit();                     // See user.c & .h

}//end InitializeSystem

/******************************************************************************
 * Function:        void USBTasks(void)
 *
 * PreCondition:    InitializeSystem has been called.
 *
 * Input:           None
 *
 * Output:          None
 *
 * Side Effects:    None
 *
 * Overview:        Service loop for USB tasks.
 *
 * Note:            None
 *****************************************************************************/
void USBTasks(void)
{
    /*
     * Servicing Hardware
     */
    

    USBCheckBusStatus();                    // Must use polling method
    if(UCFGbits.UTEYE!=1)
        USBDriverService();                 // Interrupt or polling method

}// end USBTasks

/** EOF main.c ***************************************************************/
